#include <iostream>
#include <vector>
#include "svg.h"

void
svg_begin(double width, double height) {
    std::cout << "<?xml version='1.0' encoding='UTF-8'?>\n";
    std::cout << "<svg ";
    std::cout << "width='" << width << "' ";
    std::cout << "height='" << height << "' ";
    std::cout << "viewBox='0 0 " << width << " " << height << "' ";
    std::cout << "xmlns='http://www.w3.org/2000/svg'>\n";
}

void
svg_end() {
    std::cout << "</svg>\n";
}

void
svg_text(double left, double baseline, std::string text) {
     std::cout << "<text x='" << left << "' y='"<< baseline << "' > " << text << " </text>";
}

void svg_rect(double x, double y, double width, double height){
        std::cout << "<rect x='" << x << "' y='" << y << "' width='" << width << "' height='" << height << " ' stroke='black' fill='#33A220' ></rect>";
}

double calculate_bin_height (const std::vector<std::size_t>& bins, double USER_BIN_HEIGHT, double IMAGE_HEIGHT){

    double BIN_HEIGHT;

    if ( USER_BIN_HEIGHT < 0 ){

        BIN_HEIGHT = 0;

    } else {

        BIN_HEIGHT = USER_BIN_HEIGHT;

    }



    auto USER_IMAGE_HEIGHT = bins.size() * BIN_HEIGHT;

    if (USER_IMAGE_HEIGHT > IMAGE_HEIGHT){
        BIN_HEIGHT = IMAGE_HEIGHT / bins.size();
    }

    return BIN_HEIGHT;

}

void
show_histogram_svg(const std::vector<std::size_t>& bins, double USER_BIN_HEIGHT) {

    const auto IMAGE_WIDTH = 400;
    const auto IMAGE_HEIGHT = 700;
    const auto TEXT_LEFT = 20;
    const auto TEXT_BASELINE = 20;
    const auto TEXT_WIDTH = 50;
    const auto BLOCK_WIDTH = 10;



    auto BIN_HEIGHT = calculate_bin_height (bins, USER_BIN_HEIGHT, IMAGE_HEIGHT);

    auto USER_IMAGE_HEIGHT = bins.size() * BIN_HEIGHT;


    const std::size_t MAX_ASTERISK = IMAGE_WIDTH - TEXT_WIDTH;
    svg_begin(IMAGE_WIDTH, USER_IMAGE_HEIGHT);
    double top = 0;


    std::size_t maxbin=bins[0];
    for (std::size_t i=0; i < bins.size(); i++){
        if (maxbin < bins[i]){
            maxbin=bins[i];
        }
    }

    if (maxbin<=76){
        for (std::size_t i=0; i < bins.size(); i++) {
            double bin_width = BLOCK_WIDTH * bins[i];
            svg_text(TEXT_LEFT, top + TEXT_BASELINE, std::to_string(bins[i]));
            svg_rect(TEXT_WIDTH, top, bin_width, BIN_HEIGHT);
            top += BIN_HEIGHT;
        }
        svg_end();
    } else {
        for (std::size_t i=0; i < bins.size(); i++) {
            double bin_width= MAX_ASTERISK * (static_cast<double>(bins[i]) / maxbin);
            svg_text(TEXT_LEFT, top + TEXT_BASELINE, std::to_string(bins[i]));
            svg_rect(TEXT_WIDTH, top, bin_width, BIN_HEIGHT);
            top += BIN_HEIGHT;
        }
        svg_end();
    }
}
